/*
#       lids.c --- The Linux Intrusion Detection System Configuration Tool 
#       (C) Xie Huagang 1999 All rights reserved.
#       EMail complaints to xhg@gem.ncic.ac.cn.
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
*/

#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

/*
 *
 *
 */

#define LIDS_NONE	0
#define LIDS_ADD	1
#define LIDS_DELETE	2
#define LIDS_LIST	3
#define LIDS_DELETE_ALL	4

#define LIDS_READONLY	1
#define LIDS_APPEND	2
#define LIDS_DEVICE	3

#define LIDS_EXTEND	4

#define LIDS_CONF	"/etc/lids.conf"
#define LIDS_CONF_COMMENT  "#\n# Please do not modify this file by hand\n#\n"
 

char *program;

struct lids {
	unsigned long int inode;
	unsigned int	  dev;
	int	 type;
	char 	 filename[1024];
};

struct lids lids[1024];
int	last;

void exit_error (int status, char *msg);
void exit_help();
void lids_add_file(char *filename,int type);
void lids_del_file(char *filename,int type);
void lids_del_all();
void lids_list_file(int type);
main(int argc,char **argv)
{
	int	command = LIDS_NONE;
	int 	type=LIDS_NONE;
	char 	filename[1024];
	char 	del_file[1024];
	int	c;

	program=argv[0];
    	while ((c = getopt (argc, argv, "AD:ZLr:a:d:eh")) != -1){
        	switch (c)
          	{
          	case 'A':
              		if (command != LIDS_NONE)
                 	 	exit_error (2, "multiple commands specified");
              		command = LIDS_ADD;
              		break;
          	case 'D':
              		if (command != LIDS_NONE)
                 	 	exit_error (2, "multiple commands specified");
			strcpy(del_file,optarg);
              		command = LIDS_DELETE;
              		break;
		case 'L':
              		if (command != LIDS_NONE)
                 	 	exit_error (2, "multiple commands specified");
              		command = LIDS_LIST;
              		break;
		case 'e':
              		if (command != LIDS_LIST)
                 	 	exit_error (2, "error commands specified");
              		type = LIDS_EXTEND;
			break;
		case 'Z':
              		if (command != LIDS_NONE)
                 	 	exit_error (2, "multiple commands specified");
              		command = LIDS_DELETE_ALL;
              		break;
		case 'r':
		case 'a':
		case 'd':
              		if (command == LIDS_NONE)
                 	 	exit_error (2, "no comands specified");
			if ( command == LIDS_DELETE)
                 	 	exit_error (2, "error comands specified");
			if ( type != LIDS_NONE )
                 	 	exit_error (2, "multiple types specified");
			if( c == 'r' ) type = LIDS_READONLY;
			else if( c == 'a' ) type = LIDS_APPEND;
			else if( c == 'd' ) type = LIDS_DEVICE;
			strcpy(filename,optarg);
              		break;
		case 'h':
		default:
			exit_help();	
		}
	}
	if (optind < argc)
        	exit_error (2, "unknown arguments found on commandline");
	if ( command == LIDS_NONE || argc < 2 )
		exit_help();
	if ( command == LIDS_ADD & type == LIDS_NONE) 
		exit_help();
	switch(command) {
		case LIDS_ADD:
			printf("ADD\n");
			lids_add_file(filename,type);
			break;
		case LIDS_DELETE:
			printf("DELETE\n");
			lids_del_file(del_file,type);
			break;
		case LIDS_LIST:
			printf("LIST\n");
			lids_list_file(type);
			break;
		case LIDS_DELETE_ALL:
			printf("DELETE_ALL\n");
			lids_del_all();
			break;
	}
	exit(0);
}
void lids_del_all(char *filename,int type)
{
	FILE *fd;
	
	if((fd = fopen(LIDS_CONF,"w"))== NULL ) {
		exit_error(2,"can not open conf file\n");
	}
	fwrite(LIDS_CONF_COMMENT,1,strlen(LIDS_CONF_COMMENT),fd);
	fclose(fd);
}
void lids_read_conf()
{
	FILE 	*fd;
	int	i=0,type;
	char 	buffer[1024];
	char    *p,*q;
	char 	ino_str[1024];
	char 	ino_dev[1024];
	char 	ino_filename[1024];
	
		
	if((fd = fopen(LIDS_CONF,"r"))== NULL ) {
		exit_error(2,"can not open conf file \n");
	}

	while(fgets(buffer,1024,fd) && i < 1024) {
		if ( buffer[0] == '#' ) continue;
		if ( buffer[strlen(buffer)-1] == '\n' )
			buffer[strlen(buffer)-1] = '\0' ;
		if ( buffer[0] == '[' ) {
			if ( strncmp(buffer,"[READONLY]",\
					strlen("[READONLY]"))==0 ) 
				type = LIDS_READONLY;
			else if ( strncmp(buffer,"[APPEND]",8) ==0 )  
				type = LIDS_APPEND; 
			else 
				type = LIDS_DEVICE; 
			continue;
		}
		memset(ino_str,'\0',1024);
		memset(ino_dev,'\0',1024);
		memset(ino_filename,'\0',1024);
		p = q = NULL;
		p = strchr(buffer,':');	
		if ( p == NULL ) continue; 
		strncpy(ino_str,buffer,p-buffer);
		p++;
		q = strchr(p,':');
		if ( q == NULL ) continue; 
		strncpy(ino_dev,p,q-p);
		strcpy(ino_filename,q+1);

		lids[i].inode = atol(ino_str);
		lids[i].dev = atoi(ino_dev);
		lids[i].type = type;
		strcpy(lids[i].filename,ino_filename);
#ifdef DEBUG
printf("inode = %d , dev = %d ,file=%s , type = %d \n",lids[i].inode,lids[i].dev,lids[i].filename,lids[i].type);
#endif

		i++;
	}
	last = i;

	fclose(fd);
}
void lids_write_file()
{
	int i;
	FILE *fd;
	
	if((fd = fopen(LIDS_CONF,"w"))== NULL ) {
		exit_error(2,"can not open conf file\n");
	}
	fwrite(LIDS_CONF_COMMENT,1,strlen(LIDS_CONF_COMMENT),fd);
	fprintf(fd,"[READONLY]\n");
	for(i=0;i<last;i++) {
		if ( lids[i].type == LIDS_READONLY ) 
			fprintf(fd,"%d:%d:%s\n",lids[i].inode,lids[i].dev,lids[i].filename);
	}
	fprintf(fd,"[APPEND]\n");
	for(i=0;i<last;i++) {
		if ( lids[i].type == LIDS_APPEND )
			fprintf(fd,"%d:%d:%s\n",lids[i].inode,lids[i].dev,lids[i].filename);
	}
	
	fprintf(fd,"[DEVICE]\n");
	for(i=0;i<last;i++) {
		if ( lids[i].type == LIDS_DEVICE )
			fprintf(fd,"%d:%d:%s\n",lids[i].inode,lids[i].dev,lids[i].filename);
	}

	fclose(fd);
}
void lids_list_file(int type)
{
	int i;
	lids_read_conf();
#ifdef DEBUG
	printf("last = %d \n",last);
#endif
	printf("\t \t  READ ONLY FILES \n");
	printf("\t \t  =============== \n");
	if ( type == LIDS_EXTEND) 
		printf("    Inode   \t       Dev     ");
	printf("\tFilename \n");
	printf("     --------------------------------------\n");
	for(i=0;i<last && lids[i].type == LIDS_READONLY;i++) {
		if( type == LIDS_EXTEND) {
			printf( "%10d \t%10d",
				lids[i].inode,lids[i].dev );
		}
		printf("\t%s\n",lids[i].filename);
	}	
	
	printf("\n\t\tAPPEND ONLY FILES \n");
	   printf("\t\t================= \n");
	if ( type == LIDS_EXTEND) 
		printf("    Inode   \t       Dev     ");
	printf("\tFilename \n");
	printf("    --------------------------------------\n");
	for(i=0;i<last;i++) {
		if ( lids[i].type == LIDS_APPEND )  {
			if( type == LIDS_EXTEND) {
				printf( "%10d \t%10d",
					lids[i].inode,lids[i].dev );
			}
			printf("\t%s\n",lids[i].filename);
		}
		
	}
	printf("\n\t\tProtected DEVICE MBR \n");
	   printf("\t\t================= \n");
	if ( type == LIDS_EXTEND) 
		printf("    Inode   \t       Dev     ");
	printf("\tFilename \n");
	printf("    --------------------------------------\n");
	for(i=0;i<last;i++) {
		if ( lids[i].type == LIDS_DEVICE)  {
			if( type == LIDS_EXTEND) {
				printf( "%10d \t%10d",
					lids[i].inode,lids[i].dev );
			}
			printf("\t%s\n",lids[i].filename);
		}
		
	}
	printf("\n\n");

}
/*
 *	lids_search_file()
 *
 *	lids search the file by given file and type.
 *
 */
int lids_search_file(char *filename) 
{
	int i ;

	for ( i = 0; i<last ; i++ ) {
		if ( strcmp(lids[i].filename , filename )== 0) {
			return i;
		}
	}

	return -1;
}

/*
 *	lids_search_inode()
 *
 *	lids search the file by given inode , dev  and type.
 *
 */
int lids_search_inode(struct stat st) 
{
	int i;

	for ( i = 0; i<last ; i++ ) {
		if ( lids[i].inode == st.st_ino && 
			lids[i].dev == st.st_dev ) {   
			return i;
		}
	}	
	return -1;
}
void lids_del_file(char *filename,int type)
{
	int i;
	lids_read_conf();

	i = lids_search_file(filename);
		
	if(i< 0  )
		exit_error(2,"the file does not exit in the conf file");
	lids[i].type = -1;

	lids_write_file();
}
void lids_add_file(char *filename,int type)
{
	struct stat st;

	lids_read_conf();

	if ( !stat(filename,&st) ) {
		if ( lids_search_inode(st) !=-1 )
			exit_error(2," the file which has the same inode and dev has been exited \n");
		lids[last].inode= st.st_ino;
		lids[last].dev = st.st_dev;
		lids[last].type = type;

		strcpy(lids[last].filename,filename);
		
		last++;
		lids_write_file();
	}
	else {
		exit_error(2, " can not find the file\n");
	}
}

void
exit_error (int status, char *msg)
{
    fprintf (stderr, "%s: %s\n", program, msg);
    exit (status);
}
void
exit_help ()
{
    printf ("%s v0.1 1999/10/28\n\n"
            "Usage: %s -A -[a|r|d] [file|diretory|device]\n"
            "       %s -D  file/diretoty\n"
            "       %s -Z\n"
            "       %s -L\n"
	    "       %s -h\n\n"
            "Commands:\n"
            "       -A  To add a entry\n"
            "       -D  To delelte a entry  \n"
            "       -Z  To delelte all entries \n"
            "       -L  To list all entries \n"
            "       -h  To list this help \n"
	
	    "Types:\n"
	    "	    -r	Read Only \n"
	    "	    -a  APPEND Only \n"
	    "	    -d  Protect Device MBR \n"
	
	    ,program,program,program,program,program,program);
	exit(1);
}
